#include "body.h"
#include "chip_math.h"
#include "impl.h"

Body::Body(float mass, float inertia)
{
	CHIPPER_INIT;
	cpbody = cpBodyNew(mass, inertia);
}

Body::~Body()
{
	cpBodyFree(cpbody);
}

cpBody *Body::chipmunk_body() const
{
	return cpbody;
}

void Body::set_position(const Vector2 &pos)
{
	cpbody->pos = chip_vec2(pos);
}

Vector2 Body::get_position() const
{
	return vmath_vec2(cpbody->pos);
}

void Body::set_velocity(const Vector2 &vel)
{
	cpbody->vel = chip_vec2(vel);
}

Vector2 Body::get_velocity() const
{
	return vmath_vec2(cpbody->vel);
}

void Body::set_angle(float angle)
{
	cpBodySetAngle(cpbody, angle);
}

float Body::get_angle() const
{
	return cpbody->angle;
}

void Body::set_mass(float mass)
{
	cpBodySetMass(cpbody, mass);
}

float Body::get_mass() const
{
	return cpbody->mass;
}

void Body::set_inertia(float inertia)
{
	cpBodySetMoment(cpbody, inertia);
}

float Body::get_inertia() const
{
	return cpbody->moment;
}

void Body::apply_impulse(const Vector2 &imp, const Vector2 &offs)
{
	cpBodyApplyImpulse(cpbody, chip_vec2(imp), chip_vec2(offs));
}

void Body::apply_force(const Vector2 &f, const Vector2 &offs)
{
	cpBodyApplyForce(cpbody, chip_vec2(f), chip_vec2(offs));
}

void Body::reset_forces()
{
	cpBodyResetForces(cpbody);
}

Vector2 Body::world_to_local(const Vector2 &v)
{
	return vmath_vec2(cpBodyWorld2Local(cpbody, chip_vec2(v)));
}

Vector2 Body::local_to_world(const Vector2 &v)
{
	return vmath_vec2(cpBodyLocal2World(cpbody, chip_vec2(v)));
}
