#ifndef CONSTRAINT_H_
#define CONSTRAINT_H_

#include <vmath.h>
#include "chipmunk.h"
#include "body.h"

class Constraint {
protected:
	cpConstraint *cpcon;
	Body *bodies[2];

public:
	Constraint();
	Constraint(Body *b1, Body *b2);
	virtual ~Constraint();

	cpConstraint *chipmunk_constraint() const;

	void set_bodies(Body *b1, Body *b2);
	Body *get_body(int idx) const;

	void set_max_force(float max_force);
	float get_max_force() const;

	void set_bias_coef(float bias_coef);
	float get_bias_coef() const;
};

class PivotJoint : public Constraint {
public:
	PivotJoint(const Vector2 &pivot = Vector2(0, 0));
	PivotJoint(const Vector2 &p1, const Vector2 &p2);
	PivotJoint(Body *b1, Body *b2, const Vector2 &pivot = Vector2(0, 0));
	PivotJoint(Body *b1, Body *b2, const Vector2 &p1, const Vector2 &p2);
	virtual ~PivotJoint();

	void set_pivot(const Vector2 &pivot);
	void set_pivot(const Vector2 &p1, const Vector2 &p2);
	Vector2 get_pivot(int idx = 0) const;
};

class RotLimit : public Constraint {
public:
	RotLimit(float min_rot = 0, float max_rot = 0);
	RotLimit(Body *b1, Body *b2, float min_rot = 0, float max_rot = 0);
	virtual ~RotLimit();

	void set_limits(float min_rot, float max_rot);
	float get_limit_min() const;
	float get_limit_high() const;
};

class Spring : public Constraint {
public:
	Spring(float rlen = 1, float k = 1, float damping = 0.5);
	Spring(Body *b1, Body *b2, float rlen = 1, float k = 1, float damping = 0.5);
	virtual ~Spring();

	void set_anchors(const Vector2 &a1, const Vector2 &a2);
	void set_anchor(int idx, const Vector2 &a);
	Vector2 get_anchor(int idx);

	void set_stiffness(float k);
	float get_stiffness() const;

	void set_rest_length(float rlen);
	float get_rest_length() const;

	void set_damping(float d);
	float get_damping() const;
};

#endif	// CONSTRAINT_H_
