#ifndef SHAPE_H_
#define SHAPE_H_

#include <vmath.h>
#include "chipmunk.h"
#include "body.h"

class Shape {
protected:
	cpShape *cpshape;
	Body *body;

public:
	Shape(Body *body);
	virtual ~Shape();

	cpShape *chipmunk_shape() const;

	void set_collision_type(unsigned int type);
	unsigned int get_collision_type() const;

	void set_group(unsigned int group);
	unsigned int get_group() const;

	void set_layer_mask(unsigned int mask);
	unsigned int get_layer_mask() const;

	void add_to_layer(int layer);
	void remove_from_layer(int layer);
	bool in_layer(int layer) const;

	void set_elasticity(float e);
	float get_elasticity() const;

	void set_friction(float f);
	float get_friction() const;

	void add_to_body(Body *body);
	void remove_from_body();
	Body *get_body() const;

	bool contains_point(const Vector2 &pt) const;

	virtual float calc_moment() const = 0;
};

class Circle : public Shape {
public:
	Circle(Body *body, float rad = 1.0, const Vector2 &center = Vector2(0, 0));
	virtual ~Circle();

	void set_radius(float rad);
	float get_radius() const;

	void set_center(const Vector2 &center);
	Vector2 get_center() const;

	virtual float calc_moment() const;
};

class Segment : public Shape {
public:
	Segment(Body *body, const Vector2 &v1 = Vector2(0, 0), const Vector2 &v2 = Vector2(0, 0), float rad = 0);
	virtual ~Segment();

	void set_endpoints(const Vector2 &v1, const Vector2 &v2);
	Vector2 get_endpoint(int idx) const;

	void set_radius(float r);
	float get_radius() const;

	virtual float calc_moment() const;
};

class Poly : public Shape {
public:
	Poly(Body *body, const Vector2 *varr = 0, int num_verts = 0);
	virtual ~Poly();

	void set_vertex_data(const Vector2 *varr, int num_verts);
	int get_vertex_count() const;

	virtual float calc_moment() const;
};

#endif	// SHAPE_H_
