#ifndef SIMSPACE_H_
#define SIMSPACE_H_

#include <vector>
#include <map>
#include <vmath.h>
#include "chipmunk.h"
#include "body.h"
#include "shape.h"
#include "constraint.h"

class SimSpace {
private:
	cpSpace *space;

	std::vector<Body*> bodies;
	std::vector<Shape*> shapes;
	std::vector<Constraint*> joints;

	std::map<cpShape*, Shape*> cpshape_map;
	mutable std::vector<Shape*> fres;

	float upd_interval, prev_upd;


public:
	SimSpace();
	~SimSpace();

	void set_gravity(const Vector2 &g);
	Vector2 get_gravity() const;

	void set_iterations(int iter);
	int get_iterations() const;

	void obj_size_hint(float sz);

	void add_body(Body *body);
	void add_shape(Shape *shape);
	void add_static_shape(Shape *body);
	void add_constraint(Constraint *constr);

	bool remove_body(Body *body);
	bool remove_shape(Shape *shape);
	bool remove_static_shape(Shape *shape);
	bool remove_constraint(Constraint *constr);

	void clear_bodies(void (*destr)(Body*) = 0);
	void clear_shapes(void (*destr)(Shape*) = 0);
	void clear_static_shapes(void (*destr)(Shape*) = 0);
	void clear_constraints(void (*destr)(Constraint*) = 0);

	void set_update_interval(float sec);
	void update(float sec);

	Shape **find_shapes(const Vector2 &pos, int *count) const;
};

#endif	// SIMSPACE_H_
