#include <assert.h>
#include "opengl.h"
#include "chalk.h"
#include "image.h"


static bool init();
static unsigned int load_texture(const char *fname);

static bool init_done;
static unsigned int chalk_tex, falloff_tex;

static bool init()
{
	if(!(chalk_tex = load_texture("data/chalk.png"))) {
		return false;
	}
	if(!(falloff_tex = load_texture("data/falloff3.png"))) {
		glDeleteTextures(1, (GLuint*)&chalk_tex);
		chalk_tex = 0;
		return false;
	}
	return true;
}

static unsigned int load_texture(const char *fname)
{
	unsigned int tex;
	int xsz, ysz;
	void *img;

	if(!(img = load_image(fname, &xsz, &ysz))) {
		return 0;
	}

	glGenTextures(1, (GLuint*)&tex);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, xsz, ysz, 0, GL_BGRA, GL_UNSIGNED_BYTE, img);
	free_image(img);

	if(glGetError() != GL_NO_ERROR) {
		glDeleteTextures(1, (GLuint*)&tex);
		return 0;
	}
	return tex;
}

/*static void setup_multitex(unsigned int tex0, unsigned int tex1)
{
	glActiveTextureARB(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, tex0);
	glEnable(GL_TEXTURE_2D);

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PREVIOUS);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_PREVIOUS);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA, GL_TEXTURE);

	glActiveTextureARB(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, tex1);
	glEnable(GL_TEXTURE_2D);

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_MODULATE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PREVIOUS);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_TEXTURE);
	glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA, GL_MODULATE);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA, GL_PREVIOUS);
	glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA, GL_TEXTURE);

	assert(glGetError() == GL_NO_ERROR);
}

static void disable_multitex()
{
	glActiveTextureARB(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);
}
*/

void draw_chalk_line(const PolyLine *line, float samples_per_unit)
{
	if(!init_done) {
		init();
		init_done = true;
	}

	/*setup_multitex(falloff_tex, chalk_tex);*/

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, chalk_tex);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	line->draw(samples_per_unit);

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	/*disable_multitex();*/
}

void draw_chalk_line(const Vector2 &v1, const Vector2 &v2, float width, const Color &col)
{
	PolyLine pline;
	pline.add_point(v1);
	pline.add_point(v2);
	pline.set_width(width);
	pline.set_color(col);

	draw_chalk_line(&pline, 0);
}
