#include <stdio.h>
#include <stdlib.h>
#include "opengl.h"
#include "vmath.h"
#include "image.h"
#include "parts.h"
#include "text.h"
#include "ragdoll.h"
#include "libfixgl/pixmap.h"

void draw_backdrop(float pan);
unsigned int load_texture(const char *fname);
void reshape(int x, int y);
Vector2 mouse_to_space(int x, int y);

int width, height;
float aspect;

static unsigned int back_tex;
static struct ras_pixmap back_pix, fb_pix;

unsigned int chalk_font;

Ragdoll *rag;


int main(int argc, char **argv)
{
	glutInitWindowSize(320, 240);
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	glutCreateWindow("graphics test");

	ras_init_pixmap(&fb_pix);
	ras_set_pixmap_fmt(&fb_pix, RAS_RGB16);
	fb_pix.width = 320;
	fb_pix.height = 240;
	fb_pix.bytes_per_line = 320 * fb_pix.bytes_per_pixel;

	glutReshapeFunc(reshape);

	glEnable(GL_CULL_FACE);

	// init graphics
	if(!(back_tex = load_texture("data/blackboard_pow2.jpg"))) {
		return 1;
	}

	int back_xsz, back_ysz;
	void *back_pixels = load_image("data/blackboard.jpg", &back_xsz, &back_ysz);
	if(!back_pixels) {
		return 1;
	}

	ras_init_pixmap(&back_pix);
	ras_set_pixmap_fmt(&back_pix, RAS_RGB16);
	ras_set_pixmap_pixels(&back_pix, back_xsz, back_ysz, RAS_RGBA32, back_pixels);
	free_image(back_pixels);

	if(!(chalk_font = create_font("data/chalk.ttf", 30))) {
		return 1;
	}

	rag = new Ragdoll;

	if(!init_parts()) {
		return 1;
	}
	menu_start();

	glutMainLoop();
	return 0;
}


void draw_backdrop(float pan)
{
	fb_pix.pixels = fglGetFrameBuffer();

	ras_set_pos(0, 0);
	ras_blit(&fb_pix, &back_pix, 0, 0, RAS_FULL_SIZE, RAS_FULL_SIZE);
	/*glBindTexture(GL_TEXTURE_2D, back_tex);
	glEnable(GL_TEXTURE_2D);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(pan, 0, 0);	// undo the pan
	glScalef(aspect, 1, 1);
	
	glMatrixMode(GL_TEXTURE);
	glPushMatrix();
	glLoadIdentity();
	glScalef(0.5, 1, 1);
	glTranslatef(pan / (2.0 * aspect), 0, 0);

	glBegin(GL_QUADS);
	glColor3f(1, 1, 1);
	glTexCoord2f(0, 1);
	glVertex2f(-1, -1);
	glTexCoord2f(1, 1);
	glVertex2f(1, -1);
	glTexCoord2f(1, 0);
	glVertex2f(1, 1);
	glTexCoord2f(0, 0);
	glVertex2f(-1, 1);
	glEnd();

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);*/
}

unsigned int load_texture(const char *fname)
{
	unsigned int tex;
	int xsz, ysz;
	void *img;

	if(!(img = load_image(fname, &xsz, &ysz))) {
		return 0;
	}

	glGenTextures(1, (GLuint*)&tex);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, xsz, ysz, 0, GL_BGRA, GL_UNSIGNED_BYTE, img);
	free_image(img);

	if(glGetError() != GL_NO_ERROR) {
		glDeleteTextures(1, (GLuint*)&tex);
		return 0;
	}
	return tex;
}

void reshape(int x, int y)
{
	width = x;
	height = y;
	aspect = (float)x / (float)y;

	glViewport(0, 0, x, y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0 * aspect, 1.0 * aspect, -1.0, 1.0, -1.0, 1.0);
}

Vector2 mouse_to_space(int x, int y)
{
	GLdouble model[16];
	glGetDoublev(GL_MODELVIEW_MATRIX, model);

	GLdouble proj[16];
	glGetDoublev(GL_PROJECTION_MATRIX, proj);

	GLint view[4];
	glGetIntegerv(GL_VIEWPORT, view);

	GLdouble mx, my, mz;
	int height = glutGet(GLUT_WINDOW_HEIGHT);
	gluUnProject(x, height - y, 0, model, proj, view, &mx, &my, &mz);

	return Vector2(mx, my);
}
