#include "opengl.h"
#include "parts.h"
#include "filedialog.h"
#include "level.h"
#include "message.h"
#include "palette.h"
#include "text.h"
#include "chalk.h"

#define NUM_SLOTS	8

struct FileSlot {
	bool used;
	unsigned int shot_tex;
};

static void disp();
static void idle();
static void keyb(unsigned char key, int x, int y);
static void skeyb(int key, int x, int y);
static void find_files();
static void draw_shot(int idx, float x, float y);

void draw_backdrop(float pan);
unsigned int load_texture(const char *fname);

static FileDialogMode mode;
static int sel = -1;
static char fname[512];
static FileSlot slot[NUM_SLOTS];

bool fdlg_init()
{
	return true;
}

void fdlg_start()
{
	glutDisplayFunc(disp);
	glutKeyboardFunc(keyb);
	glutSpecialFunc(skeyb);
	glutIdleFunc(idle);

	find_files();
}

void fdlg_stop()
{
	glutKeyboardFunc(0);
	glutSpecialFunc(0);
	glutIdleFunc(0);
}

void fdlg_set_mode(FileDialogMode fdlg_mode)
{
	mode = fdlg_mode;
}

FileDialogMode fdlg_get_mode()
{
	return mode;
}

const char *fdlg_get_fname()
{
	if(sel == -1) {
		return 0;
	}
	sprintf(fname, "levels/level%d.lvl", sel);
	return fname;
}

static void disp()
{
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	draw_backdrop(0.0);

	float x = 0.32;
	float y = 0.15;
	for(int i=0; i<NUM_SLOTS; i++) {
		Color col;
		if(slot[i].used) {
			col = i == sel ? get_color(CHALK_YELLOW) : get_color(CHALK_RED);
		} else {
			col = i == sel ? Color(0.7, 0.7, 0.7) : Color(0.4, 0.4, 0.4);
		}

		//float x = 0.5 - get_text_width("level 0") / 2.0;
		set_text_pos(x, y);
		set_text_color(col.x, col.y, col.z, 1.0);

		char buf[32];
		sprintf(buf, "level %d", i + 1);
		print_string(buf);

		draw_shot(i, x, y);

#ifdef GP2X
		y += get_line_advance() * 12.0;
#else
		y += get_line_advance() * 6.0;
#endif


		if(i == 3) {
			y = 0.15;
			x = 0.75;
		}
	}

	draw_msg();

	glutSwapBuffers();
}

static void idle()
{
	glutPostRedisplay();
}

static void keyb(unsigned char key, int x, int y)
{
	switch(key) {
	case 27:
		sel = -1;
	case '\n':
	case '\r':
		if(mode == FDLG_LOAD && sel != -1 && !slot[sel].used) {
			show_message("that's an unused slot");
		} else {
			switch_state(PART_EDITOR);
		}
		break;
	}
}

static void skeyb(int key, int x, int y)
{
	switch(key) {
	case GLUT_KEY_UP:
		if(sel > 0) {
			sel--;
			glutPostRedisplay();
		}
		break;

	case GLUT_KEY_DOWN:
		if(sel < NUM_SLOTS - 1) {
			sel++;
			glutPostRedisplay();
		}
		break;
	}
}

static void find_files()
{
	Level lvl;

	for(int i=0; i<NUM_SLOTS; i++) {
		sel = i;
		slot[i].shot_tex = 0;

		const char *fname = fdlg_get_fname();

		if(lvl.load(fname)) {
			slot[i].used = true;
			lvl.reset();

			char *img_fname = new char[strlen(fname) + 5];
			sprintf(img_fname, "%s.png", fname);
			slot[i].shot_tex = load_texture(img_fname);

			glGetError();	// clear any image size errors

			delete [] img_fname;

		} else {
			slot[i].used = false;
		}
	}
	sel = 0;
}

static void draw_shot(int idx, float x, float y)
{
	extern float aspect;
	y = 1.0 - 2.0 * y;
	x = aspect * (2.0 * x - 1.0);

	glPushAttrib(GL_ENABLE_BIT);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(x - 0.3, y + 0.01, 0);
	glScalef(aspect * 0.18, 0.18, 0.18);

	if(slot[idx].shot_tex) {
		glBindTexture(GL_TEXTURE_2D, slot[idx].shot_tex);
		glEnable(GL_TEXTURE_2D);
		glColor3f(1, 1, 1);
	} else {
		glColor3f(0, 0, 0);
		glDisable(GL_TEXTURE_2D);
	}

	glBegin(GL_QUADS);
	glTexCoord2f(0, 0);
	glVertex2f(-1, -1);
	glTexCoord2f(1, 0);
	glVertex2f(1, -1);
	glTexCoord2f(1, 1);
	glVertex2f(1, 1);
	glTexCoord2f(0, 1);
	glVertex2f(-1, 1);
	glEnd();

	Color col = sel == idx ? get_color(CHALK_YELLOW) : Color(1, 1, 1, 1);

	draw_chalk_line(Vector2(-1.5, -1), Vector2(1.5, -1), 0.15, col);
	draw_chalk_line(Vector2(1, -1.5), Vector2(1, 1.5), 0.15, col);
	draw_chalk_line(Vector2(1.5, 1), Vector2(-1.5, 1), 0.15, col);
	draw_chalk_line(Vector2(-1, 1.5), Vector2(-1, -1.5), 0.15, col);

	glPopMatrix();
	glPopAttrib();
}
