#ifndef LEVEL_H_
#define LEVEL_H_

#include <vector>
#include "chipper.h"
#include "polyline.h"

class Level {
private:
	std::vector<PolyLine*> lines;
	std::vector<bool> line_colid;
	std::vector<int> line_color;

	SimSpace *space;
	std::vector<Segment*> segm;
	Body *static_body;

	bool start_set, end_set;
	Vector2 start_pos, end_pos, char_pos;

	void add_line_to_space(PolyLine *line);

public:
	Level();
	~Level();

	void reset();
	bool save(const char *fname) const;
	bool load(const char *fname);

	void set_space(SimSpace *space);

	void add_line(PolyLine *line);
	void fin_line(PolyLine *line, bool colid = false, int col = 0);

	void set_char_pos(const Vector2 &pt);
	bool get_ground(const Vector2 &pt, Vector2 *gpt, Vector2 *norm) const;

	bool set_start(const Vector2 &pt);
	Vector2 get_start() const;

	bool set_end(const Vector2 &pt);
	Vector2 get_end() const;

	void draw() const;
};

#endif	// LEVEL_H_
