#include <stdlib.h>
#include "opengl.h"
#include "parts.h"
#include "palette.h"
#include "text.h"

enum {
	MENU_START_GAME,
	MENU_LEVELED,
	MENU_OPT,
	MENU_EXIT
};

static struct {
	const char *name;
	float y;
} menu[] = {
	{"start game", 0.4},
	{"edit level", 0.48},
	{"options", 0.56},
	{"exit", 0.64},
	{0, 0}
};

static const char *game_title = "ChalkRag v0.1";

static void idle();
static void disp();
static void keyb(unsigned char key, int x, int y);
static void skeyb(int key, int x, int y);
void draw_backdrop(float pan);	// chalkrag.cc

static int sel;

bool menu_init()
{
	return true;
}

void menu_start()
{
	glutDisplayFunc(disp);
	glutKeyboardFunc(keyb);
	glutSpecialFunc(skeyb);
	glutIdleFunc(idle);

	glutSetCursor(GLUT_CURSOR_NONE);
}

void menu_stop()
{
	glutKeyboardFunc(0);
	glutSpecialFunc(0);
}

static void idle()
{
	glutPostRedisplay();
}

static void disp()
{
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	draw_backdrop(0.0);

	Color col = get_color(CHALK_TEAL);
	set_text_pos(0.5 - 1.5 * get_text_width(game_title) / 2.0, 0.2);
	set_text_size(1.5);
	set_text_color(col.x, col.y, col.z, 1.0);
	print_string(game_title);

	set_text_size(1.0);
	for(int i=0; menu[i].name; i++) {
		Color color = i == sel ? get_color(CHALK_YELLOW) : get_color(CHALK_RED);
		set_text_pos(0.5 - get_text_width(menu[i].name) / 2.0, menu[i].y);
		set_text_color(color.x, color.y, color.z, 1.0);
		print_string(menu[i].name);
	}

	glutSwapBuffers();
}

static void keyb(unsigned char key, int x, int y)
{
	switch(key) {
	case ' ':
	case '\n':
	case '\r':
		switch(sel) {
		case MENU_START_GAME:
			switch_state(PART_GAME);
			break;

		case MENU_LEVELED:
			switch_state(PART_EDITOR);
			break;

		case MENU_OPT:
			switch_state(PART_OPT);
			break;

		case MENU_EXIT:
			exit(0);
		}
		break;

	case 27:
	case 'q':
		exit(0);
	}
}

static void skeyb(int key, int x, int y)
{
	switch(key) {
	case GLUT_KEY_UP:
		if(sel > 0) {
			sel--;
			glutPostRedisplay();
		}
		break;

	case GLUT_KEY_DOWN:
		if(sel < NUM_PARTS - 2) {
			sel++;
			glutPostRedisplay();
		}
		break;
	}
}
