#include <math.h>
#include "opengl.h"
#include "message.h"
#include "text.h"
#include "palette.h"

static char *msgtext;
static unsigned int msg_start, msg_dur;
static Color msg_color;
static float msg_sz = 1.5;

void show_message(const char *msg, unsigned int msec, int col)
{
	delete [] msgtext;
	msgtext = new char[strlen(msg) + 1];
	strcpy(msgtext, msg);

	msg_start = glutGet(GLUT_ELAPSED_TIME);
	msg_dur = msec;
	msg_color = col >= 0 ? get_color(col) : Color(1, 1, 1, 1);
}

static float calc_alpha(float t)
{
	if(t < 0.0 || t > 1.0) {
		return 0.0;
	} else if(t < 0.25) {
		return sin(t / 0.25 * (M_PI / 2.0));
	} else if(t > 0.75) {
		return sin((t - 0.75) / 0.25 * (M_PI / 2.0) + M_PI / 2.0);
	} else {
		return 1.0;
	}
}

void draw_msg()
{
	unsigned int msec = glutGet(GLUT_ELAPSED_TIME);

	if(msec < msg_start || msec > msg_start + msg_dur) {
		return;
	}

	float t = (float)(msec - msg_start) / (float)msg_dur;
	float alpha = calc_alpha(t);

	set_text_pos(0.5 - msg_sz * get_text_width(msgtext) / 2.0, 0.5);
	set_text_color(msg_color.x, msg_color.y, msg_color.z, alpha);
	set_text_size(msg_sz);
	print_string(msgtext);
	set_text_size(1.0);
}
