#include "parts.h"
#include "palette.h"
#include "text.h"
#include "opengl.h"

enum {
	MENU_PNAME,
	MENU_COLOR,
	MENU_BACK,

	NUM_MENU_ENTRIES
};

static struct {
	const char *name;
	float x, y;
} menu[] = {
	{"player name ", 0.275, 0.45},
	{"color ", 0.42, 0.53},
	{"back", 0.45, 0.61},
	{0, 0, 0}
};

static void disp();
static void keyb(unsigned char key, int x, int y);
static void skeyb(int key, int x, int y);
void draw_backdrop(float pan);	// chalkrag.cc

static const char *title = "Options";
static int sel;
static char *name_ptr;

#define MAX_PNAME	15
static char player_name[MAX_PNAME + 1];
static int player_color = CHALK_BLUE;

bool options_init()
{
	strcpy(player_name, "player 1");
	name_ptr = player_name;
	return true;
}

void options_start()
{
	glutDisplayFunc(disp);
	glutKeyboardFunc(keyb);
	glutSpecialFunc(skeyb);
	glutIdleFunc(0);

	glutSetCursor(GLUT_CURSOR_NONE);
}

void options_stop()
{
	glutKeyboardFunc(0);
	glutSpecialFunc(0);
}

const char *get_player_name()
{
	return player_name;
}

int get_player_color()
{
	return player_color;
}

static void disp()
{
	glClear(GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	draw_backdrop(0.0);

	Color col = get_color(CHALK_TEAL);
	set_text_pos(0.5 - 1.5 * get_text_width(title) / 2.0, 0.2);
	set_text_size(1.5);
	set_text_color(col.x, col.y, col.z, 1.0);
	print_string(title);

	set_text_size(1.0);
	for(int i=0; menu[i].name; i++) {
		Color color = i == sel ? get_color(CHALK_YELLOW) : get_color(CHALK_RED);
		set_text_pos(menu[i].x, menu[i].y);
		set_text_color(color.x, color.y, color.z, 1.0);
		print_string(menu[i].name);

		switch(i) {
		case MENU_PNAME:
			set_text_color(1, 1, 1, 1);
			print_string(player_name);
			break;

		case MENU_COLOR:
			color = get_color(player_color);
			set_text_color(color.x, color.y, color.z, 1.0);
			print_string(get_color_name(player_color));
			break;
		}
	}

	glutSwapBuffers();
}

static void keyb(unsigned char key, int x, int y)
{
	if(key == 27 || key == '\n' || key == '\r') {
		switch_state(PART_MENU);
		return;
	}

	switch(sel) {
	case MENU_PNAME:
		if(isprint(key)) {
			if(name_ptr < player_name + MAX_PNAME) {
				*name_ptr++ = key;
				*name_ptr = 0;
				glutPostRedisplay();
			}
		} else if(key == '\b' || key == 127) {
			if(name_ptr > player_name) {
				name_ptr--;
				*name_ptr = 0;
				glutPostRedisplay();
			}
		}
		break;

	case MENU_COLOR:
		if(key == ' ' || key == '\n') {
			if(player_color < NUM_CHALKS - 1) {
				player_color++;
				glutPostRedisplay();
			}
		} else if(key == '\b' || key == 127) {
			if(player_color > 0) {
				player_color--;
				glutPostRedisplay();
			}
		}
		break;

	case MENU_BACK:
		if(key == '\n' || key == '\r' || key == ' ') {
			switch_state(PART_MENU);
		}
		break;
	}
}

static void skeyb(int key, int x, int y)
{
	switch(key) {
	case GLUT_KEY_UP:
		if(sel > 0) {
			sel--;
			glutPostRedisplay();
		
			if(sel == MENU_PNAME) {
				name_ptr = player_name;
			}
		}

		break;

	case GLUT_KEY_DOWN:
		if(sel < NUM_MENU_ENTRIES - 1) {
			sel++;
			glutPostRedisplay();

			if(sel == MENU_PNAME) {
				name_ptr = player_name;
			}
		}
		break;

	case GLUT_KEY_LEFT:
		if(sel == MENU_COLOR) {
			if(player_color > 0) {
				player_color--;
				glutPostRedisplay();
			}
		}
		break;

	case GLUT_KEY_RIGHT:
		if(sel == MENU_COLOR) {
			if(player_color < NUM_CHALKS - 1) {
				player_color++;
				glutPostRedisplay();
			}
		}
		break;
	}
}
