#include "parts.h"
#include "opengl.h"

static struct Part {
	bool (*init)();
	void (*start)();
	void (*stop)();
} parts[] = {
	{menu_init, menu_start, menu_stop},
	{options_init, options_start, options_stop},
	{editor_init, editor_start, editor_stop},
	{game_init, game_start, game_stop},
	{fdlg_init, fdlg_start, fdlg_stop},
	{0, 0, 0}
};

bool init_parts()
{
	for(int i=0; parts[i].init; i++) {
		if(!parts[i].init()) {
			return false;
		}
	}
	return true;
}

#define IS_VALID(x)	((x) >= 0 && (x) < NUM_PARTS)

bool switch_state(int p)
{
	static int prev = -1;

	if(!IS_VALID(p)) {
		return false;
	}

	if(IS_VALID(prev)) {
		parts[prev].stop();
	}
	parts[p].start();

	glutPostRedisplay();

	prev = p;
	return true;
}
