#ifndef RAGDOLL_H_
#define RAGDOLL_H_

#include <vector>
#include "chipper.h"
#include "polyline.h"

class Box {
public:
	Vector2 verts[4];

	Box();
	Box(float l, float t, float r, float b);
};

enum BodyPart {
	BP_TORSO,
	BP_LU_LEG,
	BP_RU_LEG,
	BP_LL_LEG,
	BP_RL_LEG,
	BP_LU_ARM,
	BP_RU_ARM,
	BP_LL_ARM,
	BP_RL_ARM,
	BP_HEAD
};

class Ragdoll {
private:
	std::vector<Box> boxes;
	std::vector<Body*> bodies;
	std::vector<Shape*> shapes;
	std::vector<Constraint*> joints;

	Color color;

public:
	Ragdoll();
	~Ragdoll();

	void set_color(const Color &col);
	void set_position(const Vector2 &pos);
	Vector2 get_position() const;

	Body *get_body(BodyPart part);
	Shape *get_shape(BodyPart part);

	void reset();
	
	void add_to_space(SimSpace *sp) const;
	void remove_from_space(SimSpace *sp) const;

	void draw() const;
	void draw_outline() const;
};

#endif // RAGDOLL_H_
